package pesto;

import japa.parser.JavaParser;
import japa.parser.ParseException;
import japa.parser.ast.CompilationUnit;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Properties;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;

public class PageObjectTransformer {

	protected static Properties configFile;
	protected static String poFolder;
	protected static String poSikuliFolder;
	protected static String lfn;
	protected static File locFile;
	protected static String locatorsFileAsString;
	protected static Map<String, LinkedList<String>> Map_WebElement_VisualLocators = new HashMap<String, LinkedList<String>>();

	public static void main(String[] args) throws Exception {
		run();
	}

	/**
	 * run the code transformation of the Selenium Page Objects
	 * 
	 * @throws Exception
	 */
	static void run() throws Exception {
		System.out.println("[LOG] ***** STARTED PAGE OBJECTS TRANSFORMATION *****");
		getConfigProperties();
		storeVisualLocators();
		transformPageObjects();
		System.out.println("[LOG]***** ENDED PAGE OBJECTS TRANSFORMATION *****");
	}

	private static void getConfigProperties() throws IOException {

		System.out.println("[LOG]   Reading config.properties");

		configFile = new Properties();
		configFile.load(VisualLocatorGenerator.class.getClassLoader().getResourceAsStream("config.properties"));
		poFolder = configFile.getProperty("PO_SELENIUM_FOLDER");
		poSikuliFolder = configFile.getProperty("PO_SIKULI_FOLDER");
		lfn = configFile.getProperty("LOCATORS_FILE");
		locFile = new File(lfn);
		locatorsFileAsString = FileUtils.readFileToString(locFile);
	}

	// load data in Map_WebElement_VisualLocators
	private static void storeVisualLocators() throws IOException {

		System.out.println("[LOG]   Reading " + lfn);

		BufferedReader inloc = new BufferedReader(new FileReader(locFile));
		int numberOfWebElement = 0;
		int numberOfVisualLocator = 0;

		while (inloc.ready()) {
			String s = inloc.readLine();
			String multi = s.split(" -> ")[0];

			// if the WebElement has associated to more than one visual locator
			if (StringUtils.countMatches(locatorsFileAsString, multi) > 1)
			{
				// if is the first time that such WebElement has been considered =>
				// create data structure
				if (Map_WebElement_VisualLocators.get(s.split(" -> ")[0]) == null)
				{
					LinkedList<String> listWithOneElement = new LinkedList<String>();
					listWithOneElement.add(s.split(" -> ")[1]);
					Map_WebElement_VisualLocators.put(s.split(" -> ")[0], listWithOneElement);
					numberOfWebElement++;
				}
				// else simply add the new visual locator to the data structure
				else
				{
					Map_WebElement_VisualLocators.get(s.split(" -> ")[0]).add(s.split(" -> ")[1]);
					Map_WebElement_VisualLocators.put(s.split(" -> ")[0],
							Map_WebElement_VisualLocators.get(s.split(" -> ")[0]));
				}
			}
			// the WebElement has associated only one visual locator
			else
			{
				LinkedList<String> listWithOneElement = new LinkedList<String>();
				listWithOneElement.add(s.split(" -> ")[1]);
				Map_WebElement_VisualLocators.put(s.split(" -> ")[0], listWithOneElement);
				numberOfWebElement++;
			}
			numberOfVisualLocator++;
		}
		inloc.close();

		System.out.println("[LOG]   Found " + numberOfWebElement + " Web Elements and " + numberOfVisualLocator + " Visual Locators in " + lfn);
	}

	private static void transformPageObjects() throws IOException, ParseException {

		System.out.println("[LOG]   Searching for Selenium page objects in " + poFolder);

		File[] files = new File(poFolder).listFiles();

		System.out.println("[LOG]   Found " + files.length + " Selenium page objects in " + poFolder);

		for (File file : files) {

			// STEP 1 PARSE
			CompilationUnit cu;

			if (!file.getName().contains(".java"))
			{
				continue;
			}
			else
			{
				cu = parsePageObjectIntoCompilationUnit(file);
			}

			// STEP 2 TRANSFORM
			System.out.println("[LOG]   Transforming page object " + file.getName());
			modifyPageObject(cu);

			// STEP 3 SAVE
			System.out.println("[LOG]   Saving new " + file.getName() + " in " + poSikuliFolder);
			savePageObjectsOnFileSystem(cu, file);

		}
	}

	private static CompilationUnit parsePageObjectIntoCompilationUnit(File file) throws IOException,
			ParseException {

		// creates an input stream for the file to be parsed
		FileInputStream in = new FileInputStream(poFolder + file.getName());
		try
		{
			// parse the file
			CompilationUnit cu = JavaParser.parse(in);
			return cu;
		} finally
		{
			in.close();
		}
	}

	private static void modifyPageObject(CompilationUnit cu) {

		// modifies the AST
		UtilsCodeTransformation.changePackage(cu);
		UtilsCodeTransformation.addSikuliImports(cu);
		UtilsCodeTransformation.changeClass(cu, Map_WebElement_VisualLocators, locatorsFileAsString);
		UtilsCodeTransformation.changeFields(cu, locatorsFileAsString);
		UtilsCodeTransformation.removeUselessMethods(cu);
	}

	private static void savePageObjectsOnFileSystem(CompilationUnit cu, File file) throws IOException {

		File fileMod = new File(poSikuliFolder + file.getName());
		FileUtils.writeStringToFile(fileMod, cu.toString());
	}
}